//	ddprimer.cpp

#include <windows.h>
#include <windowsx.h>
#include <ddraw.h>
#include "ddprimer.h"

extern LRESULT CALLBACK WindowProcedure(
										HWND hWnd,
										UINT msg,
										WPARAM wParam,
										LPARAM lParam
										);

// case 1 default constructer
WindowsPrimer::WindowsPrimer(){}
// case 2 regular constructer
WindowsPrimer::WindowsPrimer(
							 HINSTANCE hMainInst,
							 char *szWinName,
							 WNDPROC wndproc
							)
{
	wc.style = CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc = wndproc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = (HINSTANCE)hMainInst;
	wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hIcon = NULL;
	wc.lpszClassName = szWinName;
	wc.lpszMenuName = NULL;
	if( !RegisterClass(&wc))	return;

}

// override
// create window
BOOL WindowsPrimer::windowReveal(
								 char *szTitle,
								 int nWinMode,
								 HINSTANCE hMainInst,
								 int width,
								 int height
								 )
{
	hWnd = CreateWindow(
		wc.lpszClassName, szTitle,
		// if you don't want window size be changed by the user, disable
		// the WS_THICKFRAME
		WS_VISIBLE | WS_OVERLAPPED | WS_CAPTION	| WS_THICKFRAME | WS_SYSMENU,
		0, 0,
		width + GetSystemMetrics( SM_CXSIZEFRAME) * 2,
		height + GetSystemMetrics( SM_CYSIZEFRAME) * 2
		+ GetSystemMetrics(SM_CYCAPTION),
		NULL, NULL, hMainInst, NULL);
		
	if( !hWnd)	return FALSE;
	ShowWindow( hWnd, nWinMode);
	UpdateWindow( hWnd);
	return TRUE;
}

BOOL GameWindow::initDirectDraw()
{
    /*
     * create the main DirectDraw object
     */
    DDSURFACEDESC       ddsd;
	//	creating direct draw
    if( DirectDrawCreate( NULL, &lpDD, NULL ) != DD_OK)	return FALSE;
    // Set exclusive mode
	if( lpDD->SetCooperativeLevel( hWnd, DDSCL_NORMAL ) != DD_OK)	return FALSE;// window mode
	// create primary surface
	ZeroMemory( &ddsd, sizeof( ddsd));
	ddsd.dwSize = sizeof( ddsd);
	ddsd.dwFlags = DDSD_CAPS;
	ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;
	if( lpDD->CreateSurface( &ddsd, &lpDDSPrimary, NULL) != DD_OK)	return FALSE;
	// create offscreen surface
	ZeroMemory( &ddsd, sizeof( ddsd));
	ddsd.dwSize = sizeof( ddsd);
	ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
	ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
	ddsd.dwWidth = screen_width;
	ddsd.dwHeight = screen_height;
	if( lpDD->CreateSurface( &ddsd, &lpDDSBack, NULL) != DD_OK)	return FALSE;
	// craating Direct Draw Clipper
	if( lpDD->CreateSurface( &ddsd, &lpDDSBack, NULL) != DD_OK)	return FALSE;
	// DirectDrawClipper쐬
	if( lpDD->CreateClipper(NULL, &lpClip, NULL) != DD_OK)		return FALSE;
	if( lpClip->SetHWnd( 0, hWnd) != DD_OK)	return FALSE;
	if( lpDDSPrimary->SetClipper( lpClip) != DD_OK)	return FALSE;
	// Create a timer to flip the pages
	if( !SetTimer( hWnd, TIMER_ID, TIMER_RATE, NULL)) return FALSE;
	return TRUE;
}

BOOL GameWindow::windowReveal(
								char *szTitle,
								int nWinMode,
								HINSTANCE hMainInst,
								int width,
								int height
							)
{
	screen_width = width;
	screen_height = height;
	WindowsPrimer::windowReveal(
								szTitle,
								nWinMode,
								hMainInst,
								screen_width,
								screen_height
								);
	char buf[256];
	if( initDirectDraw() != TRUE)
	{
		wsprintf(buf, "Direct Draw Init Failed (%08lx)\n", NULL );
	    MessageBox( hWnd, buf, "ERROR", MB_OK );
		finiObjects();
	    DestroyWindow( hWnd );
		return FALSE;
	}
	return TRUE;
}

// message roop
WPARAM WindowsPrimer::reaction()
{
	while( GetMessage( &msg, NULL, 0, 0))
	{
		TranslateMessage( &msg);
		DispatchMessage( &msg);
	}
	return msg.wParam;
}

HWND WindowsPrimer::getHandle(){ return hWnd; }
/*
 * finiObjects
 *
 * finished with all objects we use; release them
 */
void GameWindow::finiObjects( void )
{
#ifdef _DEBUG
	Beep( NULL, NULL);
#endif
    if( lpDD != NULL )
    {
		if( lpDDSPrimary != NULL )
		{
		    lpDDSPrimary->Release();
		    lpDDSPrimary = NULL;
		}
		lpDD->Release();
		lpDD = NULL;
    }

} /* finiObjects */

void GameWindow::render()
{
	POINT p; p.x = p.y = 0;
	RECT rc;
	// obtain the cliant area as a screen coordinate
	ClientToScreen(hWnd, &p);
	GetClientRect(hWnd, &rc);
	OffsetRect(&rc, p.x, p.y);
	// transfer to primary serface
	lpDDSPrimary->Blt(&rc, lpDDSBack, NULL, DDBLT_WAIT, NULL);
}

GameWindow::GameWindow(
					   HINSTANCE hMainInst,
					   char *szWinName,
					   WNDPROC wndproc
					   )
:WindowsPrimer( hMainInst, szWinName, wndproc)
{
	lpDD			= NULL; // DirectDraw object
	lpDDSPrimary	= NULL; // DirectDraw primary surface
	lpDDSBack		= NULL; // DirectDraw back surface
	lpClip			= NULL;
}

bool GameWindow::activateHDC()
{
	if ( lpDDSBack->GetDC(&hdc) == DD_OK)	return true;
	else									return false;
}
void GameWindow::disactivateHDC()
{
	lpDDSBack->ReleaseDC(hdc);
}
// Before calling this function, activateHDC() should be called.
bool GameWindow::clearScreen()
{
	RECT	rc;
	if( hdc != NULL)
	{
		GetClientRect( hWnd, &rc);
		FillRect( hdc, &rc, (HBRUSH)GetStockObject( BLACK_BRUSH));
		return true;
	}
	else
	{
		return false;
	}
}
// Before calling this function, activateHDC() should be called.
bool GameWindow::drawLine( COLORREF color, int x1, int y1, int x2, int y2)
{
	if( hdc != NULL)
	{
		HPEN hPen, hOldPen;
		hPen = CreatePen( PS_SOLID, 5, color);
		hOldPen = (HPEN)SelectObject( hdc, hPen);
		MoveToEx( hdc, x1, y1, NULL);
		LineTo(hdc, x2, y2);
		SelectObject( hdc, hOldPen);
		DeleteObject( hPen);
		return true;
	}
	return false;
}
// Before calling this function, activateHDC() should have been called.
bool GameWindow::textOut( int x, int y, char *str)
{
	if( hdc != NULL)
	{
		SetBkColor( hdc, RGB( 0, 0, 255 ) );
		SetTextColor( hdc, RGB( 255, 255, 0 ) );
		TextOut( hdc, x, y, str, lstrlen(str) );
		return true;
	}
	return false;
}

